@@ -61,7 +61,7 @@ if config.CA_IMPL == 'cfssl':
 elif config.CA_IMPL == 'openssl':
     from keylime import ca_impl_openssl as ca_impl
 else:
-    raise Exception("Unknown CA implementation: %s" % config.CA_IMPL)
+    raise Exception(f"Unknown CA implementation: {config.CA_IMPL}")
 
 
 
@@ -98,7 +98,7 @@ def cmd_mkcert(workingdir, name):
         cert, pk = ca_impl.mk_signed_cert(
             cacert, ca_pk, name, priv[0]['lastserial'] + 1)
 
-        with open('%s-cert.crt' % name, 'wb') as f:
+        with open(f'{name}-cert.crt', 'wb') as f:
             f.write(cert.public_bytes(serialization.Encoding.PEM))
 
         priv[0][name] = pk.private_bytes(
@@ -112,16 +112,16 @@ def cmd_mkcert(workingdir, name):
 
         write_private(priv)
 
-        with os.fdopen(os.open("%s-private.pem" % name, os.O_WRONLY | os.O_CREAT, 0o600), 'wb') as f:
+        with os.fdopen(os.open(f"{name}-private.pem", os.O_WRONLY | os.O_CREAT, 0o600), 'wb') as f:
             f.write(priv[0][name])
 
-        with os.fdopen(os.open("%s-public.pem" % name, os.O_WRONLY | os.O_CREAT, 0o600), 'wb') as f:
+        with os.fdopen(os.open(f"{name}-public.pem", os.O_WRONLY | os.O_CREAT, 0o600), 'wb') as f:
             f.write(pk.public_key().public_bytes(
                 encoding=serialization.Encoding.PEM,
                 format=serialization.PublicFormat.SubjectPublicKeyInfo
             ))
 
-        cc = load_cert_by_path('%s-cert.crt' % name)
+        cc = load_cert_by_path(f'{name}-cert.crt')
         pubkey = cacert.public_key()
         pubkey.verify(
             cc.signature,
@@ -130,7 +130,7 @@ def cmd_mkcert(workingdir, name):
             cc.signature_hash_algorithm,
         )
 
-        logger.info(f"Created certificate for name {name} successfully in {workingdir}")
+        logger.info("Created certificate for name %s successfully in %s", name, workingdir)
     except crypto_exceptions.InvalidSignature:
         logger.error("ERROR: Cert does not validate against CA")
     finally:
@@ -194,7 +194,7 @@ def cmd_init(workingdir):
             cac.signature_hash_algorithm,
         )
 
-        logger.info(f"CA certificate created successfully in {workingdir}")
+        logger.info("CA certificate created successfully in %s", workingdir)
     except crypto_exceptions.InvalidSignature:
         logger.error("ERROR: Cert does not self validate")
     finally:
@@ -254,7 +254,7 @@ def cmd_certpkg(workingdir, name, insecure=False):
                 f.write(pkg)
         else:
             # actually output the package to disk with a protected private key
-            with zipfile.ZipFile('%s-pkg.zip' % name, 'w', compression=zipfile.ZIP_STORED) as f:
+            with zipfile.ZipFile(f'{name}-pkg.zip', 'w', compression=zipfile.ZIP_STORED) as f:
                 f.writestr(f"{name}-public.pem", pub)
                 f.writestr(f"{name}-cert.crt", cert)
                 f.writestr(f"{name}-private.pem", prot_priv)
@@ -262,8 +262,8 @@ def cmd_certpkg(workingdir, name, insecure=False):
                 f.writestr('cacrl.der', crl)
                 f.writestr('cacrl.pem', crlpem)
 
-        logger.info("Creating cert package for %s in %s-pkg.zip" %
-                    (name, name))
+        logger.info("Creating cert package for %s in %s-pkg.zip",
+                    name, name)
 
         return pkg, serial, subject
     finally:
@@ -292,7 +292,7 @@ def get_crl_distpoint(cert_path):
 
     except x509.extensions.ExtensionNotFound:
         pass
-    logger.info(f"No CRL distribution points in {cert_path}")
+    logger.info("No CRL distribution points in %s", cert_path)
     return ""
 
 # to check: openssl crl -inform DER -text -noout -in cacrl.der
@@ -377,13 +377,13 @@ def cmd_listen(workingdir, cert_path):
         serveraddr = ('', config.CRL_PORT)
         server = ThreadedCRLServer(serveraddr, CRLHandler)
         if os.path.exists('cacrl.der'):
-            logger.info("Loading existing crl: %s" %
+            logger.info("Loading existing crl: %s",
                         os.path.abspath("cacrl.der"))
             with open('cacrl.der', 'rb') as f:
                 server.setcrl(f.read())
         t = threading.Thread(target=server.serve_forever)
-        logger.info("Hosting CRL on %s:%d" %
-                    (socket.getfqdn(), config.CRL_PORT))
+        logger.info("Hosting CRL on %s:%d",
+                    socket.getfqdn(), config.CRL_PORT)
         t.start()
 
         def check_expiration():
@@ -404,7 +404,7 @@ def cmd_listen(workingdir, cert_path):
                                 in1hour = datetime.datetime.utcnow() + datetime.timedelta(hours=6)
                                 if expire <= in1hour:
                                     logger.info(
-                                        "Certificate to expire soon %s, re-issuing" % expire)
+                                        "Certificate to expire soon %s, re-issuing", expire)
                                     cmd_regencrl(workingdir)
                     # check a little less than every hour
                     time.sleep(3540)
@@ -422,10 +422,10 @@ def cmd_listen(workingdir, cert_path):
             json_meta = json.loads(revocation['meta_data'])
             serial = json_meta['cert_serial']
             if revocation.get('type', None) != 'revocation' or serial is None:
-                logger.error("Unsupported revocation message: %s" % revocation)
+                logger.error("Unsupported revocation message: %s", revocation)
                 return
 
-            logger.info("Revoking certificate: %s" % serial)
+            logger.info("Revoking certificate: %s", serial)
             server.setcrl(cmd_revoke(workingdir, None, serial))
         try:
             while True:
@@ -454,7 +454,7 @@ class ThreadedCRLServer(ThreadingMixIn, HTTPServer):
 
 class CRLHandler(BaseHTTPRequestHandler):
     def do_GET(self):
-        logger.info('GET invoked from ' + str(self.client_address) + ' with uri:' + self.path)
+        logger.info('GET invoked from %s with uri: %s', str(self.client_address), self.path)
 
         if self.server.published_crl is None:
             self.send_response(404)
@@ -475,7 +475,6 @@ def rmfiles(path):
 def write_private(inp):
     priv = inp[0]
     salt = inp[1]
-    global global_password
 
     priv_encoded = yaml.dump(priv, Dumper=SafeDumper)
     key = crypto.kdf(global_password, salt)
@@ -487,7 +486,6 @@ def write_private(inp):
 
 
 def read_private(warn=False):
-    global global_password
     if global_password is None:
         setpassword(getpass.getpass(
             "Please enter the password to decrypt your keystore: "))
@@ -505,14 +503,14 @@ def read_private(warn=False):
 
     if warn:
         # file doesn't exist, just invent a salt
-        logger.warning("Private certificate data %s does not exist yet." %
+        logger.warning("Private certificate data %s does not exist yet.",
                        os.path.abspath("private.yml"))
         logger.warning(
             "Keylime will attempt to load private certificate data again when it is needed.")
     return {'revoked_keys': []}, base64.b64encode(crypto.generate_random_key()).decode()
 
 
-def main(argv=sys.argv):
+def main(argv=sys.argv):  #pylint: disable=dangerous-default-value
     parser = argparse.ArgumentParser(argv[0])
     parser.add_argument('-c', '--command', action='store', dest='command',
                         required=True, help="valid commands are init,create,pkg,revoke,listen")
@@ -526,9 +524,10 @@ def main(argv=sys.argv):
     args = parser.parse_args(argv[1:])
 
     if args.dir is None:
-        if os.getuid() != 0 and config.REQUIRE_ROOT:
+        if not os.access(config.WORK_DIR, os.R_OK + os.W_OK):
             logger.error(
-                "If you don't specify a working directory, this process must be run as root to access %s" % config.WORK_DIR)
+                "If you don't specify a working directory, this process must be run as a user with access to %s",
+                config.WORK_DIR)
             sys.exit(-1)
         workingdir = config.CA_WORK_DIR
     else:
@@ -563,10 +562,10 @@ def main(argv=sys.argv):
     elif args.command == 'listen':
         if args.name is None:
             args.name = os.path.join(workingdir, 'RevocationNotifier-cert.crt')
-            logger.warning("using default name for revocation cert %s"
-                           % args.name)
+            logger.warning("using default name for revocation cert %s",
+                           args.name)
         cmd_listen(workingdir, args.name)
     else:
-        logger.error("Invalid command: %s" % args.command)
+        logger.error("Invalid command: %s", args.command)
         parser.print_help()
         sys.exit(-1)
