@@ -9,6 +9,8 @@ import sys
 import functools
 import asyncio
 import os
+from multiprocessing import Process
+
 
 from sqlalchemy.exc import SQLAlchemyError
 from sqlalchemy.orm.exc import NoResultFound
@@ -17,7 +19,6 @@ import tornado.web
 
 from keylime import config
 from keylime import json
-from keylime import registrar_client
 from keylime.agentstates import AgentAttestStates
 from keylime.common import states, validators, retry
 from keylime.db.verifier_db import VerfierMain
@@ -77,7 +78,6 @@ def _from_db_obj(agent_db_obj):
                 'operational_state', \
                 'public_key', \
                 'tpm_policy', \
-                'vtpm_policy', \
                 'meta_data', \
                 'mb_refstate', \
                 'allowlist', \
@@ -101,6 +101,11 @@ def _from_db_obj(agent_db_obj):
     agent_dict = {}
     for field in fields:
         agent_dict[field] = getattr(agent_db_obj, field, None)
+
+    # add default fields that are ephemeral
+    for key,val in exclude_db.items():
+        agent_dict[key] = val
+
     return agent_dict
 
 
@@ -122,7 +127,7 @@ def store_attestation_state(agentAttestState):
             ima_pcrs_dict = agentAttestState.get_ima_pcrs()
             update_agent.ima_pcrs = list(ima_pcrs_dict.keys())
             for pcr_num, value in ima_pcrs_dict.items():
-                setattr(update_agent, 'pcr%d' % pcr_num, value)
+                setattr(update_agent, f'pcr{pcr_num}', value)
             update_agent.learned_ima_keyrings = agentAttestState.get_ima_keyrings().to_json()
             try:
                 session.add(update_agent)
@@ -253,8 +258,7 @@ class AgentsHandler(BaseHandler):
                 self, 405, "Not Implemented: Use /agents/ interface")
             return
 
-        if not rest_params["api_version"]:
-            web_util.echo_json_response(self, 400, "API Version not supported")
+        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
             return
 
         if "agents" not in rest_params:
@@ -285,10 +289,10 @@ class AgentsHandler(BaseHandler):
                 web_util.echo_json_response(self, 404, "agent id not found")
         else:
             json_response = None
-            if "bulk" in rest_params.keys():
+            if "bulk" in rest_params:
                 agent_list = None
 
-                if ("verifier" in rest_params.keys()) and (rest_params["verifier"] != ''):
+                if ("verifier" in rest_params) and (rest_params["verifier"] != ''):
                     agent_list = session.query(VerfierMain).filter_by(verifier_id=rest_params["verifier"]).all()
                 else:
                     agent_list = session.query(VerfierMain).all()
@@ -299,7 +303,7 @@ class AgentsHandler(BaseHandler):
 
                 web_util.echo_json_response(self, 200, "Success", json_response)
             else:
-                if ("verifier" in rest_params.keys()) and (rest_params["verifier"] != ''):
+                if ("verifier" in rest_params) and (rest_params["verifier"] != ''):
                     json_response = session.query(VerfierMain.agent_id).filter_by(
                         verifier_id=rest_params["verifier"]).all()
                 else:
@@ -323,8 +327,7 @@ class AgentsHandler(BaseHandler):
                 self, 405, "Not Implemented: Use /agents/ interface")
             return
 
-        if not rest_params["api_version"]:
-            web_util.echo_json_response(self, 400, "API Version not supported")
+        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
             return
 
         if "agents" not in rest_params:
@@ -399,8 +402,7 @@ class AgentsHandler(BaseHandler):
                     self, 405, "Not Implemented: Use /agents/ interface")
                 return
 
-            if not rest_params["api_version"]:
-                web_util.echo_json_response(self, 400, "API Version not supported")
+            if not web_util.validate_api_version(self, rest_params["api_version"], logger):
                 return
 
             if "agents" not in rest_params:
@@ -432,7 +434,6 @@ class AgentsHandler(BaseHandler):
                     agent_data['operational_state'] = states.START
                     agent_data['public_key'] = ""
                     agent_data['tpm_policy'] = json_body['tpm_policy']
-                    agent_data['vtpm_policy'] = json_body['vtpm_policy']
                     agent_data['meta_data'] = json_body['metadata']
                     agent_data['allowlist'] = json_body['allowlist']
                     agent_data['mb_refstate'] = json_body['mb_refstate']
@@ -442,6 +443,8 @@ class AgentsHandler(BaseHandler):
                     agent_data['accept_tpm_encryption_algs'] = json_body['accept_tpm_encryption_algs']
                     agent_data['accept_tpm_signing_algs'] = json_body['accept_tpm_signing_algs']
                     agent_data['supported_version'] = json_body['supported_version']
+                    agent_data['ak_tpm'] = json_body['ak_tpm']
+                    agent_data['mtls_cert'] = json_body.get('mtls_cert', None)
                     agent_data['hash_alg'] = ""
                     agent_data['enc_alg'] = ""
                     agent_data['sign_alg'] = ""
@@ -455,22 +458,8 @@ class AgentsHandler(BaseHandler):
                     agent_data['verifier_ip'] = config.get('cloud_verifier', 'cloudverifier_ip')
                     agent_data['verifier_port'] = config.get('cloud_verifier', 'cloudverifier_port')
 
-                    # We fetch the registrar data directly here because we require it for connecting to the agent
-                    # using mTLS
-                    registrar_client.init_client_tls('cloud_verifier')
-                    registrar_data = registrar_client.getData(config.get("cloud_verifier", "registrar_ip"),
-                                                              config.get("cloud_verifier", "registrar_port"), agent_id)
-                    if registrar_data is None:
-                        web_util.echo_json_response(self, 400,
-                                                    f"Data for agent {agent_id} could not be found in registrar!")
-                        logger.warning(f"Data for agent {agent_id} could not be found in registrar!")
-                        return
-
-                    agent_data['mtls_cert'] = registrar_data.get('mtls_cert', None)
-                    agent_data['ak_tpm'] = registrar_data['aik_tpm']
-
                     # TODO: Always error for v1.0 version after initial upgrade
-                    if registrar_data.get('mtls_cert', None) is None and agent_data['supported_version'] != "1.0":
+                    if agent_data['mtls_cert'] is None and agent_data['supported_version'] != "1.0":
                         web_util.echo_json_response(self, 400, "mTLS certificate for agent is required!")
                         return
 
@@ -491,7 +480,7 @@ class AgentsHandler(BaseHandler):
 
                     if new_agent_count > 0:
                         web_util.echo_json_response(
-                            self, 409, "Agent of uuid %s already exists" % (agent_id))
+                            self, 409, f"Agent of uuid {agent_id} already exists")
                         logger.warning("Agent of uuid %s already exists", agent_id)
                     else:
                         try:
@@ -502,16 +491,16 @@ class AgentsHandler(BaseHandler):
                             logger.error('SQLAlchemy Error: %s', e)
                             raise e
 
-                        for key in list(exclude_db.keys()):
-                            agent_data[key] = exclude_db[key]
+                        # add default fields that are ephemeral
+                        for key,val in exclude_db.items():
+                            agent_data[key] = val
 
                         # Prepare SSLContext for mTLS connections
-                        # TODO: drop special handling after initial upgrade
-                        mtls_cert = registrar_data.get('mtls_cert', None)
+                        agent_mtls_cert_enabled = config.getboolean('cloud_verifier', 'agent_mtls_cert_enabled', fallback=False)
+                        mtls_cert = agent_data['mtls_cert']
                         agent_data['ssl_context'] = None
-                        if mtls_cert:
-                            agent_data['ssl_context'] = web_util.generate_agent_mtls_context(mtls_cert,
-                                                                                             self.mtls_options)
+                        if agent_mtls_cert_enabled and mtls_cert:
+                            agent_data['ssl_context'] = web_util.generate_agent_mtls_context(mtls_cert, self.mtls_options)
 
                         if agent_data['ssl_context'] is None:
                             logger.warning('Connecting to agent without mTLS: %s', agent_id)
@@ -524,7 +513,7 @@ class AgentsHandler(BaseHandler):
                 web_util.echo_json_response(self, 400, "uri not supported")
                 logger.warning("POST returning 400 response. uri not supported")
         except Exception as e:
-            web_util.echo_json_response(self, 400, "Exception error: %s" % e)
+            web_util.echo_json_response(self, 400, f"Exception error: {str(e)}")
             logger.warning("POST returning 400 response. Exception error: %s", e)
             logger.exception(e)
 
@@ -542,8 +531,7 @@ class AgentsHandler(BaseHandler):
                     self, 405, "Not Implemented: Use /agents/ interface")
                 return
 
-            if not rest_params["api_version"]:
-                web_util.echo_json_response(self, 400, "API Version not supported")
+            if not web_util.validate_api_version(self, rest_params["api_version"], logger):
                 return
 
             if "agents" not in rest_params:
@@ -604,7 +592,7 @@ class AgentsHandler(BaseHandler):
                 logger.warning("PUT returning 400 response. uri not supported")
 
         except Exception as e:
-            web_util.echo_json_response(self, 400, "Exception error: %s" % e)
+            web_util.echo_json_response(self, 400, f"Exception error: {str(e)}")
             logger.warning("PUT returning 400 response. Exception error: %s", e)
             logger.exception(e)
 
@@ -628,15 +616,14 @@ class AllowlistHandler(BaseHandler):
             web_util.echo_json_response(self, 400, "Invalid URL")
             return
 
-        if not rest_params["api_version"]:
-            web_util.echo_json_response(self, 400, "API Version not supported")
+        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
             return
 
         allowlist_name = rest_params['allowlists']
         if allowlist_name is None:
             web_util.echo_json_response(self, 400, "Invalid URL")
             logger.warning(
-                'GET returning 400 response: ' + self.request.path)
+                'GET returning 400 response: %s', self.request.path)
             return
 
         session = get_session()
@@ -644,15 +631,15 @@ class AllowlistHandler(BaseHandler):
             allowlist = session.query(VerifierAllowlist).filter_by(
                 name=allowlist_name).one()
         except NoResultFound:
-            web_util.echo_json_response(self, 404, "Allowlist %s not found" % allowlist_name)
+            web_util.echo_json_response(self, 404, f"Allowlist {allowlist_name} not found")
             return
         except SQLAlchemyError as e:
-            logger.error(f'SQLAlchemy Error: {e}')
+            logger.error('SQLAlchemy Error: %s', e)
             web_util.echo_json_response(self, 500, "Failed to get allowlist")
             raise
 
         response = {}
-        for field in ('name', 'tpm_policy', 'vtpm_policy', 'ima_policy'):
+        for field in ('name', 'tpm_policy', 'ima_policy'):
             response[field] = getattr(allowlist, field, None)
         web_util.echo_json_response(self, 200, 'Success', response)
 
@@ -667,15 +654,14 @@ class AllowlistHandler(BaseHandler):
             web_util.echo_json_response(self, 400, "Invalid URL")
             return
 
-        if not rest_params["api_version"]:
-            web_util.echo_json_response(self, 400, "API Version not supported")
+        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
             return
 
         allowlist_name = rest_params['allowlists']
         if allowlist_name is None:
             web_util.echo_json_response(self, 400, "Invalid URL")
             logger.warning(
-                'DELETE returning 400 response: ' + self.request.path)
+                'DELETE returning 400 response: %s', self.request.path)
             return
 
         session = get_session()
@@ -683,10 +669,10 @@ class AllowlistHandler(BaseHandler):
             session.query(VerifierAllowlist).filter_by(
                 name=allowlist_name).one()
         except NoResultFound:
-            web_util.echo_json_response(self, 404, "Allowlist %s not found" % allowlist_name)
+            web_util.echo_json_response(self, 404, f"Allowlist {allowlist_name} not found")
             return
         except SQLAlchemyError as e:
-            logger.error(f'SQLAlchemy Error: {e}')
+            logger.error('SQLAlchemy Error: %s', e)
             web_util.echo_json_response(self, 500, "Failed to get allowlist")
             raise
 
@@ -695,7 +681,7 @@ class AllowlistHandler(BaseHandler):
                 name=allowlist_name).delete()
             session.commit()
         except SQLAlchemyError as e:
-            logger.error(f'SQLAlchemy Error: {e}')
+            logger.error('SQLAlchemy Error: %s', e)
             web_util.echo_json_response(self, 500, "Failed to get allowlist")
             raise
 
@@ -705,13 +691,13 @@ class AllowlistHandler(BaseHandler):
         self.set_header('Content-Type', 'application/json')
         self.finish()
         logger.info(
-            'DELETE returning 204 response for allowlist: ' + allowlist_name)
+            'DELETE returning 204 response for allowlist: %s', allowlist_name)
 
     def post(self):
         """Create an allowlist
 
         POST /allowlists/{name}
-        body: {"tpm_policy": {..}, "vtpm_policy": {..}
+        body: {"tpm_policy": {..} ...
         """
 
         rest_params = web_util.get_restful_params(self.request.uri)
@@ -719,8 +705,7 @@ class AllowlistHandler(BaseHandler):
             web_util.echo_json_response(self, 400, "Invalid URL")
             return
 
-        if not rest_params["api_version"]:
-            web_util.echo_json_response(self, 400, "API Version not supported")
+        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
             return
 
         allowlist_name = rest_params['allowlists']
@@ -742,9 +727,6 @@ class AllowlistHandler(BaseHandler):
         tpm_policy = json_body.get('tpm_policy')
         if tpm_policy:
             allowlist['tpm_policy'] = tpm_policy
-        vtpm_policy = json_body.get('vtpm_policy')
-        if vtpm_policy:
-            allowlist['vtpm_policy'] = vtpm_policy
         ima_policy = json_body.get('ima_policy')
         if ima_policy:
             allowlist['ima_policy'] = ima_policy
@@ -756,12 +738,12 @@ class AllowlistHandler(BaseHandler):
                 VerifierAllowlist).filter_by(name=allowlist_name).count()
             if al_count > 0:
                 web_util.echo_json_response(
-                    self, 409, "Allowlist with name %s already exists" % allowlist_name)
+                    self, 409, f"Allowlist with name {allowlist_name} already exists")
                 logger.warning(
-                    "Allowlist with name %s already exists" % allowlist_name)
+                    "Allowlist with name %s already exists", allowlist_name)
                 return
         except SQLAlchemyError as e:
-            logger.error(f'SQLAlchemy Error: {e}')
+            logger.error('SQLAlchemy Error: %s', e)
             raise
 
         try:
@@ -769,7 +751,7 @@ class AllowlistHandler(BaseHandler):
             session.add(VerifierAllowlist(**allowlist))
             session.commit()
         except SQLAlchemyError as e:
-            logger.error(f'SQLAlchemy Error: {e}')
+            logger.error('SQLAlchemy Error: %s', e)
             raise
 
         web_util.echo_json_response(self, 201)
@@ -796,14 +778,15 @@ async def invoke_get_quote(agent, need_pubkey):
     # TODO: remove special handling after initial upgrade
     if agent['ssl_context']:
         res = tornado_requests.request("GET",
-                                       "https://%s:%d/v%s/quotes/integrity?nonce=%s&mask=%s&vmask=%s&partial=%s&ima_ml_entry=%d" %
-                                       (agent['ip'], agent['port'], agent['supported_version'], params["nonce"], params["mask"], params['vmask'], partial_req, params['ima_ml_entry']),
+                                       f"https://{agent['ip']}:{agent['port']}/v{agent['supported_version']}/quotes/integrity"
+                                       f"?nonce={params['nonce']}&mask={params['mask']}"
+                                       f"&partial={partial_req}&ima_ml_entry={params['ima_ml_entry']}",
                                        context=agent['ssl_context'])
     else:
         res = tornado_requests.request("GET",
-                                       "http://%s:%d/v%s/quotes/integrity?nonce=%s&mask=%s&vmask=%s&partial=%s&ima_ml_entry=%d" %
-                                       (agent['ip'], agent['port'], agent['supported_version'], params["nonce"], params["mask"],
-                                        params['vmask'], partial_req, params['ima_ml_entry']))
+                                       f"http://{agent['ip']}:{agent['port']}/v{agent['supported_version']}/quotes/integrity"
+                                       f"?nonce={params['nonce']}&mask={params['mask']}"
+                                       f"&partial={partial_req}&ima_ml_entry={params['ima_ml_entry']}")
     response = await res
 
     if response.status_code != 200:
@@ -854,11 +837,11 @@ async def invoke_provide_v(agent):
     # TODO: remove special handling after initial upgrade
     if agent['ssl_context']:
         res = tornado_requests.request(
-            "POST", "https://%s:%d/v%s/keys/vkey" % (agent['ip'], agent['port'], agent['supported_version']),
+            "POST", f"https://{agent['ip']}:{agent['port']}/v{agent['supported_version']}/keys/vkey",
             data=v_json_message, context=agent['ssl_context'])
     else:
         res = tornado_requests.request(
-            "POST", "http://%s:%d/v%s/keys/vkey" % (agent['ip'], agent['port'], agent['supported_version']),
+            "POST", f"http://{agent['ip']}:{agent['port']}/v{agent['supported_version']}/keys/vkey",
             data=v_json_message)
 
     response = await res
@@ -1050,18 +1033,12 @@ async def activate_agents(verifier_id, verifier_ip, verifier_port, mtls_options)
             if agent.boottime:
                 ima_pcrs_dict = {}
                 for pcr_num in agent.ima_pcrs:
-                    ima_pcrs_dict[pcr_num] = getattr(agent, 'pcr%d' % pcr_num)
+                    ima_pcrs_dict[pcr_num] = getattr(agent, f'pcr{pcr_num}')
                 aas.add(agent.agent_id, agent.boottime, ima_pcrs_dict, agent.next_ima_ml_entry, agent.learned_ima_keyrings)
         session.commit()
     except SQLAlchemyError as e:
         logger.error('SQLAlchemy Error: %s', e)
 
-def start_tornado(tornado_server, port):
-    tornado_server.listen(port)
-    print("Starting Torando on port " + str(port))
-    tornado.ioloop.IOLoop.instance().start()
-    print("Tornado finished")
-
 
 def main():
     """Main method of the Cloud Verifier Server.  This method is encapsulated in a function for packaging to allow it to be
@@ -1121,27 +1098,58 @@ def main():
     sockets = tornado.netutil.bind_sockets(
         int(cloudverifier_port), address=cloudverifier_host)
 
-    tornado.process.fork_processes(config.getint(
-        'cloud_verifier', 'multiprocessing_pool_num_workers'))
+    def server_process(task_id):
+        logger.info("Starting server of process %s", task_id)
+        engine.dispose()
+        server = tornado.httpserver.HTTPServer(app, ssl_options=context, max_buffer_size=max_upload_size)
+        server.add_sockets(sockets)
 
-    server = tornado.httpserver.HTTPServer(app, ssl_options=context, max_buffer_size=max_upload_size)
-    server.add_sockets(sockets)
+        def server_sig_handler(*_):
+            logger.info("Shutting down server %s..", task_id)
+            # Stop server to not accept new incoming connections
+            server.stop()
 
-    signal.signal(signal.SIGTERM, lambda *_: sys.exit(0))
+            # Wait for all connections to be closed and then stop ioloop
+            async def stop():
+                await server.close_all_connections()
+                tornado.ioloop.IOLoop.current().stop()
+            asyncio.ensure_future(stop())
+
+        # Attach signal handler to ioloop.
+        # Do not use signal.signal(..) for that because it does not work!
+        loop = asyncio.get_event_loop()
+        loop.add_signal_handler(signal.SIGINT, server_sig_handler)
+        loop.add_signal_handler(signal.SIGTERM, server_sig_handler)
 
-    try:
         server.start()
-        if tornado.process.task_id() == 0:
-            # Start the revocation notifier only on one process
-            if config.getboolean('cloud_verifier', 'revocation_notifier'):
-                logger.info("Starting service for revocation notifications on port %s",
-                            config.getint('cloud_verifier', 'revocation_notifier_port'))
-                revocation_notifier.start_broker()
-            # Auto activate agents
+        if task_id == 0:
+            # Reactivate agents
             asyncio.ensure_future(activate_agents(cloudverifier_id, cloudverifier_host, cloudverifier_port, mtls_options))
-
         tornado.ioloop.IOLoop.current().start()
-    except (KeyboardInterrupt, SystemExit):
-        tornado.ioloop.IOLoop.current().stop()
-        if tornado.process.task_id() == 0 and config.getboolean('cloud_verifier', 'revocation_notifier'):
+        logger.debug("Server %s stopped.", task_id)
+        sys.exit(0)
+
+    processes = []
+
+    def sig_handler(*_):
+        if config.getboolean('cloud_verifier', 'revocation_notifier'):
             revocation_notifier.stop_broker()
+        for p in processes:
+            p.join()
+        sys.exit(0)
+
+    signal.signal(signal.SIGINT, sig_handler)
+    signal.signal(signal.SIGTERM, sig_handler)
+    if config.getboolean('cloud_verifier', 'revocation_notifier'):
+        logger.info("Starting service for revocation notifications on port %s",
+                    config.getint('cloud_verifier', 'revocation_notifier_port'))
+        revocation_notifier.start_broker()
+
+    num_workers = config.getint(
+        'cloud_verifier', 'multiprocessing_pool_num_workers')
+    if num_workers <= 0:
+        num_workers = tornado.process.cpu_count()
+    for task_id in range(0, num_workers):
+        process = Process(target=server_process, args=(task_id,))
+        process.start()
+        processes.append(process)
