@@ -27,7 +27,7 @@ from keylime import api_version as keylime_api_version
 
 logger = keylime_logging.init_logging('tenant_webapp')
 tenant_templ = tenant.Tenant()
-(my_cert, my_priv_key), agent_cert = tenant_templ.get_tls_context()
+(my_cert, my_priv_key), agent_cert, _ = tenant_templ.get_tls_context()
 cert = (my_cert, my_priv_key)
 if config.getboolean('general', "enable_tls"):
     tls_enabled = True
@@ -108,12 +108,9 @@ class WebAppHandler(BaseHandler):
         will return errors.
         """
 
-        # Get default policies for TPM/vTPM from config as suggestions to user
+        # Get default policies for TPM from config as suggestions to user
         tpm_policy = json.dumps(json.loads(
             config.get('tenant', 'tpm_policy')), indent=2)
-        vtpm_policy = json.dumps(json.loads(
-            config.get('tenant', 'vtpm_policy')), indent=2)
-
         # Get default intervals for populating angents, updating agents and updating terminal
         populate_agents_interval = json.dumps(json.loads(
             config.get('webapp', 'populate_agents_interval')), indent=2)
@@ -125,7 +122,7 @@ class WebAppHandler(BaseHandler):
         self.set_status(200)
         self.set_header('Content-Type', 'text/html')
         self.write(
-            """
+            f"""
             <!DOCTYPE html>
             <html>
                 <head>
@@ -140,9 +137,9 @@ class WebAppHandler(BaseHandler):
                                 droppable[i].addEventListener('drop', fileUploadCallback, false);
                             }}
                             populateAgents();
-                            setInterval(populateAgents, {0});
-                            setInterval(updateAgentsInfo, {1});
-                            setInterval(updateTerminal, {2});
+                            setInterval(populateAgents, {populate_agents_interval});
+                            setInterval(updateAgentsInfo, {update_agents_interval});
+                            setInterval(updateTerminal, {update_terminal_interval});
                         }}
                     </script>
                     <link href='/static/css/webapp.css' rel='stylesheet' type='text/css'/>
@@ -150,11 +147,11 @@ class WebAppHandler(BaseHandler):
                 <body>
                     <div id='modal_box' onclick="if (event.target == this) {{toggleVisibility(this.id);resetAddAgentForm();return false;}}">
 
-            """.format(populate_agents_interval, update_agents_interval, update_terminal_interval)
+            """
         )
 
         self.write(
-            """
+            f"""
                         <div id='modal_body'>
                             <center>
                                 <h3>Add Agent</h3>
@@ -193,37 +190,31 @@ class WebAppHandler(BaseHandler):
                                 </div>
                                 <br>
 
-                                <div id='policy_toggle' onclick="toggleVisibility('policy_block');" title='TPM &amp; vTPM Policy Configuration'>
-                                    TPM &amp; vTPM Policy Configuration
+                                <div id='policy_toggle' onclick="toggleVisibility('policy_block');" title='TPM Policy Configuration'>
+                                    TPM Policy Configuration
                                 </div>
                                 <div id="policy_block">
                                     <div class="form_block">
                                         <label for='tpm_policy'>TPM Policy: </label><br>
-                                        <textarea class='json_input' id='tpm_policy' name='tpm_policy'>{}</textarea>
-                                        <br>
-                                    </div>
-
-                                    <div class="form_block">
-                                        <label for='vtpm_policy'>vTPM Policy: </label><br>
-                                        <textarea class='json_input' id='vtpm_policy' name='vtpm_policy'>{}</textarea>
+                                        <textarea class='json_input' id='tpm_policy' name='tpm_policy'>{tpm_policy}</textarea>
                                         <br>
                                     </div>
                                 </div>
                                 <br>
-            """.format(tpm_policy, vtpm_policy)
+            """
         )
 
         self.write(
-            """
+            f"""
                                 <div id="payload_block">
                                     <div class="form_block">
                                         <label for='ptype'>Payload type: </label>
-                                        <label><input type='radio' name='ptype' value='{}' checked="checked" onclick='toggleTabs(this.value)'> File </label>&nbsp;
-                                        <label><input type='radio' name='ptype' value='{}' onclick='toggleTabs(this.value)'> Keyfile </label>&nbsp;
-                                        <label><input type='radio' name='ptype' value='{}' onclick='toggleTabs(this.value)'> CA Dir </label>&nbsp;
+                                        <label><input type='radio' name='ptype' value='{Agent_Init_Types.FILE}' checked="checked" onclick='toggleTabs(this.value)'> File </label>&nbsp;
+                                        <label><input type='radio' name='ptype' value='{Agent_Init_Types.KEYFILE}' onclick='toggleTabs(this.value)'> Keyfile </label>&nbsp;
+                                        <label><input type='radio' name='ptype' value='{Agent_Init_Types.CA_DIR}' onclick='toggleTabs(this.value)'> CA Dir </label>&nbsp;
                                         <br>
                                     </div>
-            """.format(Agent_Init_Types.FILE, Agent_Init_Types.KEYFILE, Agent_Init_Types.CA_DIR)
+            """
         )
 
         self.write(
@@ -539,13 +530,10 @@ class AgentsHandler(BaseHandler):
             logger.warning('POST returning 400 response. malformed query')
             return
 
-        # Pull in user-defined v/TPM policies
+        # Pull in user-defined TPM policies
         tpm_policy = self.get_argument("tpm_policy", "", True)
         if tpm_policy == "":
             tpm_policy = None
-        vtpm_policy = self.get_argument("vtpm_policy", "", True)
-        if vtpm_policy == "":
-            vtpm_policy = None
 
         # Pull in allowlist
         allowlist = None
@@ -573,7 +561,6 @@ class AgentsHandler(BaseHandler):
             'incl_dir': incl_dir,
             'ca_dir_pw': ca_dir_pw,
             'tpm_policy': tpm_policy,
-            'vtpm_policy': vtpm_policy,
             'allowlist': allowlist,
             'ima_exclude': ima_exclude,
         }
@@ -584,8 +571,8 @@ class AgentsHandler(BaseHandler):
             mytenant.agent_uuid = agent_id
             mytenant.init_add(args)
             mytenant.preloop()
-            mytenant.do_cv()
             mytenant.do_quote()
+            mytenant.do_cv()
         except Exception as e:
             logger.exception(e)
             logger.warning('POST returning 500 response. Tenant error: %s', e)
@@ -682,8 +669,7 @@ def get_tls_context():
     context.load_cert_chain(
         certfile=my_tls_cert, keyfile=my_tls_priv_key)
     context.verify_mode = ssl.CERT_REQUIRED
-    context.check_hostname = config.getboolean(
-        'general', 'tls_check_hostnames')
+    context.check_hostname = False
     return context
 
 
@@ -693,10 +679,6 @@ def main():
 
     webapp_port = config.getint('webapp', 'webapp_port')
 
-    if not config.REQUIRE_ROOT and webapp_port < 1024:
-        webapp_port += 2000
-        logger.warning("Running without root, changing port to %d", webapp_port)
-
     logger.info('Starting Tenant WebApp (tornado) on port %d use <Ctrl-C> to stop', webapp_port)
 
     # Figure out where our static files are located
@@ -708,7 +690,7 @@ def main():
         root_dir = os.path.dirname(os.path.abspath(__file__))
     if not os.path.exists(root_dir + "/static/"):
         raise Exception(
-            'Static resource directory could not be found in %s!' % (root_dir))
+            f'Static resource directory could not be found in {root_dir}!')
 
     app = tornado.web.Application([
         (r"/webapp/.*", WebAppHandler),
@@ -721,7 +703,6 @@ def main():
 
     # WebApp Server TLS
     server_context = get_tls_context()
-    server_context.check_hostname = False  # config.getboolean('general', 'tls_check_hostnames')
     server_context.verify_mode = ssl.CERT_NONE  # ssl.CERT_REQUIRED
 
     # Set up server
