@@ -13,9 +13,6 @@ Tests all but two RESTful interfaces:
     * CV's PUT /agents/{UUID}
         - POST already bootstraps agent, so PUT is redundant in this test
 
-The registrar's PUT vactivate interface is only tested if a vTPM is present!
-
-
 USAGE:
 Should be run in test directory under root privileges with either command:
     * python -m unittest -v test_restful
@@ -58,6 +55,8 @@ from keylime import api_version
 from keylime.common import algorithms
 
 
+#pylint: disable=no-self-use
+
 # Coverage support
 if "COVERAGE_FILE" in os.environ:
     FORK_ARGS = ["coverage", "run", "--parallel-mode"]
@@ -86,8 +85,8 @@ def cmp(a, b):
 
 
 # Ensure this is run as root
-if os.geteuid() != 0 and config.REQUIRE_ROOT:
-    sys.exit("Tests need to be run with root privileges, or set env KEYLIME_TEST=True!")
+if os.geteuid() != 0:
+    sys.exit("Tests need to be run with root privileges")
 
 # Force sorting tests alphabetically
 unittest.TestLoader.sortTestMethodsUsing = lambda _, x, y: cmp(x, y)
@@ -108,7 +107,6 @@ mtls_cert = None
 keyblob = None
 ek_tpm = None
 aik_tpm = None
-vtpm = False
 
 # Set up mTLS
 my_cert = config.get('tenant', 'my_cert')
@@ -133,10 +131,10 @@ def setUpModule():
         env = os.environ.copy()
         env['PATH'] = env['PATH'] + ":/usr/local/bin"
         # Run init_tpm_server and tpm_serverd (start fresh)
-        its = subprocess.Popen(["init_tpm_server"], shell=False, env=env)
-        its.wait()
-        tsd = subprocess.Popen(["tpm_serverd"], shell=False, env=env)
-        tsd.wait()
+        with subprocess.Popen(["init_tpm_server"], shell=False, env=env) as its:
+            its.wait()
+        with subprocess.Popen(["tpm_serverd"], shell=False, env=env) as tsd:
+            tsd.wait()
     except Exception:
         print("WARNING: Restarting TPM emulator failed!")
     # Note: the following is required as abrmd is failing to reconnect to MSSIM, once
@@ -188,7 +186,7 @@ def setUpModule():
     tenant_templ.agent_base_url = f'{tenant_templ.cloudagent_ip}:{tenant_templ.cloudagent_port}'
     tenant_templ.supported_version = "2.0"
     # Set up TLS
-    tenant_templ.cert, tenant_templ.agent_cert = tenant_templ.get_tls_context()
+    tenant_templ.cert, tenant_templ.agent_cert, _ = tenant_templ.get_tls_context()
 
 
 # Destroy everything on teardown
@@ -201,9 +199,9 @@ def tearDownModule():
 
 def launch_cloudverifier():
     """Start up the cloud verifier"""
-    global cv_process, script_env, FORK_ARGS
+    global cv_process
     if cv_process is None:
-        cv_process = subprocess.Popen("keylime_verifier",
+        cv_process = subprocess.Popen("keylime_verifier",  # pylint: disable=subprocess-popen-preexec-fn,consider-using-with
                                       shell=False,
                                       preexec_fn=os.setsid,
                                       stdout=subprocess.PIPE,
@@ -228,9 +226,9 @@ def launch_cloudverifier():
 
 def launch_registrar():
     """Start up the registrar"""
-    global reg_process, script_env, FORK_ARGS
+    global reg_process
     if reg_process is None:
-        reg_process = subprocess.Popen("keylime_registrar",
+        reg_process = subprocess.Popen("keylime_registrar",  # pylint: disable=subprocess-popen-preexec-fn,consider-using-with
                                        shell=False,
                                        preexec_fn=os.setsid,
                                        stdout=subprocess.PIPE,
@@ -255,7 +253,7 @@ def launch_registrar():
 
 def launch_cloudagent(agent="python"):
     """Start up the cloud agent"""
-    global agent_process, script_env, FORK_ARGS
+    global agent_process
     if agent == "python":
         agent_path = "keylime_agent"
     elif agent == "rust":
@@ -263,7 +261,7 @@ def launch_cloudagent(agent="python"):
     else:
         agent_path = "echo"
     if agent_process is None:
-        agent_process = subprocess.Popen(agent_path,
+        agent_process = subprocess.Popen(agent_path,  # pylint: disable=subprocess-popen-preexec-fn,consider-using-with
                                          shell=False,
                                          preexec_fn=os.setsid,
                                          stdout=subprocess.PIPE,
@@ -337,7 +335,6 @@ class TestRestful(unittest.TestCase):
     payload = None
     auth_tag = None
     tpm_policy = {}
-    vtpm_policy = {}
     metadata = {}
     allowlist = {}
     revocation_key = ""
@@ -364,9 +361,7 @@ class TestRestful(unittest.TestCase):
 
         # Prepare policies for agent
         cls.tpm_policy = config.get('tenant', 'tpm_policy')
-        cls.vtpm_policy = config.get('tenant', 'vtpm_policy')
         cls.tpm_policy = tpm_abstract.TPM_Utilities.readPolicy(cls.tpm_policy)
-        cls.vtpm_policy = tpm_abstract.TPM_Utilities.readPolicy(cls.vtpm_policy)
 
         # Allow targeting a specific API version (default latest)
         cls.api_version = '2.0'
@@ -382,7 +377,7 @@ class TestRestful(unittest.TestCase):
     # Registrar Testset
     def test_010_reg_agent_post(self):
         """Test registrar's POST /agents/{UUID} Interface"""
-        global keyblob, vtpm, tpm_instance, ek_tpm, aik_tpm
+        global keyblob, tpm_instance, ek_tpm, aik_tpm
         contact_ip = "127.0.0.1"
         contact_port = 9002
         tpm_instance = tpm_main.tpm()
@@ -401,13 +396,10 @@ class TestRestful(unittest.TestCase):
         # Initialize the TPM with AIK
         (ekcert, ek_tpm, aik_tpm) = tpm_instance.tpm_init(self_activate=False,
                                                            config_pw=config.get('cloud_agent', 'tpm_ownerpassword'))
-        vtpm = tpm_instance.is_vtpm()
 
-        # Handle virtualized and emulated TPMs
+        # Handle emulated TPMs
         if ekcert is None:
-            if vtpm:
-                ekcert = 'virtual'
-            elif tpm_instance.is_emulator():
+            if tpm_instance.is_emulator():
                 ekcert = 'emulator'
 
         # Get back to our original CWD
@@ -441,10 +433,8 @@ class TestRestful(unittest.TestCase):
         keyblob = json_response["results"]["blob"]
         self.assertIsNotNone(keyblob, "Malformed response body!")
 
-    @unittest.skipIf(vtpm, "Registrar's PUT /agents/{UUID}/activate only for non-vTPMs!")
     def test_011_reg_agent_activate_put(self):
         """Test registrar's PUT /agents/{UUID}/activate Interface"""
-        global keyblob
 
         self.assertIsNotNone(keyblob, "Required value not set.  Previous step may have failed?")
 
@@ -508,6 +498,8 @@ class TestRestful(unittest.TestCase):
         self.assertIn("port", json_response["results"], "Malformed response body!")
 
         global aik_tpm
+        global mtls_cert
+        mtls_cert = json_response["results"]["mtls_cert"]
         aik_tpm = json_response["results"]["aik_tpm"]
 
     def test_015_reg_agent_delete(self):
@@ -598,7 +590,6 @@ class TestRestful(unittest.TestCase):
         """Test agent's POST /keys/vkey Interface"""
         # CV should do this (during CV POST/PUT test)
         # Running this test might hide problems with the CV sending the V key
-        global public_key
 
         self.assertIsNotNone(self.V, "Required value not set.  Previous step may have failed?")
         self.assertIsNotNone(public_key, "Required value not set.  Previous step may have failed?")
@@ -623,7 +614,6 @@ class TestRestful(unittest.TestCase):
 
     def test_024_agent_keys_ukey_post(self):
         """Test agents's POST /keys/ukey Interface"""
-        global public_key
 
         self.assertIsNotNone(public_key, "Required value not set.  Previous step may have failed?")
         self.assertIsNotNone(self.U, "Required value not set.  Previous step may have failed?")
@@ -657,7 +647,6 @@ class TestRestful(unittest.TestCase):
         data = {
             'name': 'test-allowlist',
             'tpm_policy': json.dumps(self.tpm_policy),
-            'vtpm_policy': json.dumps(self.vtpm_policy),
             'ima_policy': json.dumps(self.allowlist),
         }
 
@@ -692,7 +681,6 @@ class TestRestful(unittest.TestCase):
         results = json_response['results']
         self.assertEqual(results['name'], 'test-allowlist')
         self.assertEqual(results['tpm_policy'], json.dumps(self.tpm_policy))
-        self.assertEqual(results['vtpm_policy'], json.dumps(self.vtpm_policy))
         self.assertEqual(results['ima_policy'], json.dumps(self.allowlist))
 
     def test_027_cv_allowlist_delete(self):
@@ -718,7 +706,6 @@ class TestRestful(unittest.TestCase):
             'cloudagent_ip': tenant_templ.cloudagent_ip,
             'cloudagent_port': tenant_templ.cloudagent_port,
             'tpm_policy': json.dumps(self.tpm_policy),
-            'vtpm_policy': json.dumps(self.vtpm_policy),
             'allowlist': json.dumps(self.allowlist),
             'ima_sign_verification_keys': '',
             'mb_refstate': None,
@@ -727,7 +714,9 @@ class TestRestful(unittest.TestCase):
             'accept_tpm_hash_algs': config.get('tenant', 'accept_tpm_hash_algs').split(','),
             'accept_tpm_encryption_algs': config.get('tenant', 'accept_tpm_encryption_algs').split(','),
             'accept_tpm_signing_algs': config.get('tenant', 'accept_tpm_signing_algs').split(','),
-            'supported_version': tenant_templ.supported_version
+            'supported_version': tenant_templ.supported_version,
+            'ak_tpm': aik_tpm,
+            'mtls_cert': mtls_cert,
         }
 
         test_030_cv_agent_post = RequestsClient(tenant_templ.verifier_base_url, tls_enabled)
@@ -815,7 +804,6 @@ class TestRestful(unittest.TestCase):
             'cloudagent_ip': tenant_templ.cloudagent_ip,
             'cloudagent_port': tenant_templ.cloudagent_port,
             'tpm_policy': json.dumps(self.tpm_policy),
-            'vtpm_policy': json.dumps(self.vtpm_policy),
             'allowlist': json.dumps(allowlist),
             'ima_sign_verification_keys': '',
             'metadata': json.dumps(self.metadata),
@@ -850,7 +838,6 @@ class TestRestful(unittest.TestCase):
 
         nonce = tpm_abstract.TPM_Utilities.random_password(20)
         mask = self.tpm_policy["mask"]
-        vmask = self.vtpm_policy["mask"]
         partial = "1"
         if public_key is None:
             partial = "0"
@@ -858,7 +845,7 @@ class TestRestful(unittest.TestCase):
         test_040_agent_quotes_integrity_get = RequestsClient(tenant_templ.agent_base_url,
                                                              tls_enabled=True, ignore_hostname=True)
         response = test_040_agent_quotes_integrity_get.get(
-            f'/v{self.api_version}/quotes/integrity?nonce={nonce}&mask={mask}&vmask={vmask}&partial={partial}',
+            f'/v{self.api_version}/quotes/integrity?nonce={nonce}&mask={mask}&partial={partial}',
             cert=tenant_templ.agent_cert,
             verify=False  # TODO: use agent certificate
         )
